/**
 * \file xic_base.cpp
 * \date 21 sept. 2009
 * \author Olivier Langella
 */

#include <algorithm>


#include "xic_base.h"
#include "../quanti_items/quantiItemBase.h"

xicBase::xicBase(const Msrun * p_ms_run,
		 const pappso::MassRange & mz_range,  pappso::XicExtractMethod xic_type)
    :
    _p_ms_run(p_ms_run), _mz_range(mz_range) ,_xic_type(xic_type)
{
}

xicBase::xicBase(const xicBase & xic_base):
    pappso::Xic(xic_base),
    _p_ms_run(xic_base._p_ms_run), _mz_range(xic_base._mz_range), _xic_type(xic_base._xic_type)
{
}


xicBase::~xicBase() {
}

const Msrun *
xicBase::getMsRun() const {
    return (_p_ms_run);
}


std::vector<mcq_double>
xicBase::getAlignedRetentionTimes() const {
    std::vector<mcq_double> p_v_aligned_rt;
    auto it = _v_xic_elements.begin();
    auto itend = _v_xic_elements.end();
    for (; it != itend; ++it) {
        p_v_aligned_rt.push_back(_p_ms_run->getAlignedRtByOriginalRt(it->rt));
    }
    return (p_v_aligned_rt);
}

const vector< mcq_double > xicBase::getIntensities() const {
    std::vector<mcq_double> p_v_int;
    auto it = _v_xic_elements.begin();
    auto itend = _v_xic_elements.end();
    for (; it != itend; ++it) {
        p_v_int.push_back(it->intensity);
    }
    return (p_v_int);
}


void xicBase::fillDataArray(mcq_double * xdata, mcq_double * ydata, unsigned int plotsize) const {
    auto it = _v_xic_elements.begin();
    auto itend = _v_xic_elements.end();
    for (size_t i=0; it != itend; ++it,++i) {
        xdata[i] = it->rt;
        ydata[i] = it->intensity;
    }
}

void xicBase::applyFilter(const pappso::XicFilterInterface & filter) {
  Xic newxic;
  filter.filter(*this, &newxic);
  _v_xic_elements.resize(newxic.size());
  copy(newxic.begin(),newxic.end(),_v_xic_elements.begin());
}


void
xicBase::toStream(QTextStream & to_stream) const {


    auto it = _v_xic_elements.begin();
    auto itend = _v_xic_elements.end();
    for (; it != itend; ++it) {
        to_stream << it->rt << " ";
    }
    to_stream << endl;

    it = _v_xic_elements.begin();
    itend = _v_xic_elements.end();
    for (; it != itend; ++it) {
        to_stream << it->intensity << " ";
    }
    to_stream << endl;
}
