
/*******************************************************************************
 * Copyright (c) 2016 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 *
 * This file is part of the MassChroQ software.
 *
 *     MassChroQ is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     MassChroQ is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with MassChroQ.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contributors:
 *     Olivier Langella <Olivier.Langella@moulon.inra.fr> - initial API and implementation
 ******************************************************************************/
#ifndef XICFILTERDETECTMATCHREALRT_H
#define XICFILTERDETECTMATCHREALRT_H

#include <QMutex>
#include "../monitors/monitorspeedinterface.h"
#include <pappsomspp/xic/filters/xicfilterinterface.h>
#include <pappsomspp/xic/detection/xicdetectioninterface.h>
#include "../quantificator.h"
#include "xictracebase.h"

class XicFilterDetectMatchRealRt: public pappso::XicDetectionSinkInterface
{
public:
    XicFilterDetectMatchRealRt(XicTraceBase * p_trace, MonitorSpeedInterface & monitor, const pappso::XicFilterInterface * p_xic_filter,const pappso::XicDetectionInterface * p_detection, Quantificator * p_quantificator, McqMatchingMode matching_mode);
    ~XicFilterDetectMatchRealRt();

    virtual void setXicPeak (pappso::XicPeak & xic_peak) override;

    void filterDetectQuantify(const pappso::Xic * xic, const std::vector<MsMsRtIntensity> * p_real_rt,  QuantiItemBase * p_currentSearchItem, const Msrun * p_msrun);

private :
    MonitorSpeedInterface & _monitor;
    Quantificator * _p_quantificator;
    const pappso::XicDetectionInterface * _p_detection;
    const pappso::XicFilterInterface * _p_xic_filter;
    std::map<pappso::XicPeakSp, unsigned int> _peak_counter_map;

    const std::vector<MsMsRtIntensity> * _p_real_rt;
    QuantiItemBase * _p_current_search_item = nullptr;
    const Msrun * _p_current_msrun;
    
    std::vector<pappso::XicPeakSp> * _p_log_peaks = nullptr;
    XicTraceBase * _p_trace=nullptr;
    McqMatchingMode _matching_mode;
    
    //QMutex _mutex;
};

#endif // XICFILTERDETECTMATCHREALRT_H
